# frozen_string_literal: true
require 'user_management/utils/group_diff'
require 'cht-lib-fabric/contexts/request'
require 'user_management/utils/event_publish'

module UserManagement
  module Actions
    class UpdateIdentityAttributes
      include Import['logger', 'user_management.kafka_message', 'alert']
      include UserManagement::Utils::EventPublish
      
      def call(customer_id, user_id, raw_info)
        # Call out to BE here to persistently store attributes
        # TODO: Uncomment this when we are ready for a dynamo db connection
        # dynamo = UserManagement::Utils::RawInfoDynamoDbWrapper.new
        # dynamo.set_data(customer_id, "#{user_id}#{raw_info['email']}", raw_info)
        # dynamo.get_data(customer_id, "#{user_id}#{raw_info['email']}")
        customer = ::UserManagement::Stores::ArModels::Customer.find(customer_id)

        result = customer.using_data_shard do
          redacted_info = raw_info.slice('roles', 'groups')
          raw_info.keys.each do |key|
            redacted_info[key] = '**redacted**' unless redacted_info.key?(key)
          end
          # Save off former user groups
          user = ::UserManagement::Stores::ArModels::User.find(user_id)
          former_groups = user.user_groups.to_a          

          logger.info "IdP attributes for user=#{user_id}, customer=#{customer_id}: #{redacted_info}"
          membership_updater = ::UserManagement::Utils::MembershipUpdater.new(customer, user, raw_info)
          membership_updater.update_memberships

          # Save off current groups
          current_groups = user.user_groups.to_a
          publish_group_changes(customer_id, user_id, former_groups, current_groups)
        end

        {user_id: user_id, group_ids: result}
      end

      # 
      # Publish group changes during SSO membership
      #
      # @param - customer_id      - The integer value of a customer ID
      # @param - user_id          - The integer value of a user ID
      # @param - former_groups    - An array of user groups the user was in prior to membership computation
      # @param - current_grorups  - An array of user groups the user is in after membership computation 
      def publish_group_changes(customer_id, user_id, former_groups, current_groups)
        changed_groups = []
        begin
          # Manufacture a context for passing as we dont have one here, nor do
          # we have an organization, but we need for portion of the message
          ctx = get_context_for_message(customer_id, user_id, nil)
          # The list of groups the user was removed from or added to
          changed_groups = ::UserManagement::Utils::GroupDiff.new.diff(former_groups, current_groups)

          logger.info("Generating #{changed_groups.size} user group change messages for user #{user_id}")

          # Publish Kafka Event indicating user group(s) were changed
          changed_groups.each do |group|
            publish(group, ctx)
          end
        rescue => ex
          logger.error "Unable to publish update_user_idp groups event #{ex.message}"
          alert.exception(
            ex,
            customer_id: customer_id,
            user_id: user_id,
            organization_id: nil,
            action: ::UserManagement::KafkaMessage::CHANGE
          )
          # returning an empty array here as I'm not sure we want to delay group assignment
          # due to a kafka failure. So we'll rollbar and return
          []
        end
        changed_groups
      end

      # 
      # Manufacture a request context, this request comes in
      # through a REST endpoint, the majority of publishers run inside
      # a graphql mutation which have a request context provided
      #
      # @param - customer_id     - The integer value of a customer ID
      # @param - user_id         - The integer value of a user ID
      # @param - organization_id - The integer value of an organization ID
      def get_context_for_message(customer_id, user_id, organization_id) 
        req = Platform::Contexts::Request.new
        req.current_customer = customer_id
        req.current_organization = organization_id
        req.current_user = user_id
        { :request => req }
      end
    end
  end
end
