require 'cht-lib-fabric/helpers/graphql/raise_error'
require 'cht-lib-fabric/error_codes'

module UserManagement
  class Authorizer
    extend ::Platform::Helpers::GraphQL::RaiseError

    # This method is used to enforce authorization in an action.
    #
    # @raise [Platform::GraphQL::Errors::ExecutionError] if the user is not authorized.
    # @return [true] if the user is authorized
    def self.authorization
      Application['clients.authorization']
    end

    def self.logger
      Application['logger']
    end

    def self.authorize!(subject:, actions:, require_all: false, context: {})
      # if we have been given a valid token (validated by signed token)
      # with from a service account and we can skip authorization
      service_account = begin
                          jwt = context[:request]&.jwt_claims
                          jwt_sub = jwt ? jwt[:sub] : nil
                          if jwt_sub
                            ::Platform::CRNs::ServiceAccount.from_s(jwt_sub)
                            true
                          else
                            false
                          end
                        rescue ::Platform::InvalidCRNError
                          false
                        end
      if service_account
        logger.info "Bypassing authorization for service account for #{subject} and action: #{actions}"
        return true
      end

      enforcement = if require_all
                      Platform::Clients::Authorization::ENFORCEMENT_ALL
                    else
                      Platform::Clients::Authorization::ENFORCEMENT_ANY
                    end

      subject = "crn::#{subject}/*"
      logger.info "Checking authorization for #{subject}, #{actions} with enforcement #{enforcement}"
      authorized = authorization.authorized(
        subject: subject,
        actions: Array(actions),
        enforcement: enforcement,
        context: context
      )
      logger.info "authorized? #{authorized}"

      raise_error(
        ::Platform::ErrorCodes::NOT_AUTHORIZED,
        "Not authorized",
        uuid: context[:request].request_id
      ) unless authorized

      true
    end
  end
end
