# frozen_string_literal: true

require 'graphql'
require 'import'
require 'active_record'

module UserManagement
  module GraphQL
    module Types
      # Mixin module for a field extension which:
      # * Adds a single argument field to its parent graphql type
      # * Uses this argument to mutate the ActiveRecord::Relation containing the result set
      #
      # In practice, this is probably just SortableExtension and FilterableExtension
      #
      # Requires a constant in the including module namespace:
      # * ARGUMENT_FIELD
      #
      # Provides two helper instance methods:
      # * argument_field (a reflection of the ARGUMENT_FIELD constant)
      # * relation_field_from_input
      #   (encapsulates some logic to avoid SQL injection on user-provided input fields)
      #
      # To work, requires an instance method to be implemented by includers:
      # * with_ar_condition
      #   Takes the original ActiveRecord::Relation result and applies a single
      #   rule to it, and returns the result
      module ActiveRecordWrapperFieldExtension
        def self.included(mod)
          arg_field = mod.const_get('ARGUMENT_FIELD')
          raise NoMethodError, "Missing constant ARGUMENT_FIELD in #{mod}" if arg_field.nil?

          mod.define_method(:argument_field, ->() { arg_field })
        end

        # Method which must be implemented by subclass.
        # Takes the existing ActiveRecord::Relation and a single mutator argument
        # and returns a mutated ActiveRecord::Relation.
        # For example, this could take something like
        #   User.where(customer_id: 12)
        # and apply a sort rule like
        #   { field: "name", direction: DESC }
        # and returns
        #   User.where(customer_id: 12).order(name: :desc)
        def with_ar_condition(relation, input_rule)
          raise NotImplementedError, "Must be implemented by subclass"
        end

        # Method called automatically when resolving a GraphQL request on this type
        def resolve(object:, arguments:, context:)
          # Remove rule arguments before passing along
          result = yield(object, arguments.reject { |k| k == argument_field })
          return result if arguments[argument_field].nil?

          unless result.is_a?(::ActiveRecord::Relation)
            logger.warn("Attempt to process a non-active record object; skipping")
            return result
          end

          # Apply ActiveRecord mutations to the result one at a time
          # by delegating to the subclass's `with_ar_condition` method
          arguments[argument_field].reduce(result) do |ar_relation, rule|
            with_ar_condition(ar_relation, rule)
          end
        end

        # Basic support for some fields we have recently added 
        # that dont have a 1-1 mapping between GraphQL field and AR field
        # Ideally this should be done at the GraphQL field level but GraphQL
        # ruby only appears to offer aliasing at the argument level
        def translate_field_name(field_name)
          translated_field_names =
            {
              "lastActivity" => "lastRequestAt",
              "lastSignIn"  => "CurrentSignInAt"
            }
          unless translated_field_names[field_name].nil?
            return translated_field_names[field_name]
          end
          return field_name
        end

        # Helper method to get a DB-safe field name from user input.
        # Also translates from GraphQL-like field names to snake case.
        def relation_field_from_input(relation:, field:)
          # Check for a translated field name
          field = translate_field_name(field)
          field_name = ::GraphQL::Schema::Member::BuildType.underscore(field)
          unless relation.column_names.include?(field_name)
            raise ::GraphQL::ExecutionError, "Unknown field #{field_name}"
          end

          "#{relation.table_name}.#{field_name}"
        end
      end
    end
  end
end
