# frozen_string_literal: true

require 'graphql'
require 'import'
require 'active_record'

module UserManagement
  module GraphQL
    module Types
      class FilterType < ::GraphQL::Schema::Enum
        FILTER_TYPES = %w{
          EQ
          NE
          GT
          LT
          GTE
          LTE
          MATCHES
          NOT_MATCHES
          NULL
          NOT_NULL
        }
        FILTER_TYPES.each do |filter_type|
          const_set filter_type, filter_type
        end

        value EQ, "Field matches the value input identically"
        value NE, "Field can be anything which does not identically match the value input"
        value GT, <<~EOQ
          Field is larger than the given value input.
          Behavior depends on the underlying type of the field
          and may not work as expected for some fields.
        EOQ
        value LT, <<~EOQ
          Field is less than the given value input.
          Behavior depends on the underlying type of the field
          and may not work as expected for some fields.
        EOQ
        value GTE, <<~EOQ
          Field is larger than or identical to the given value input.
          Behavior depends on the underlying type of the field
          and may not work as expected for some fields.
        EOQ
        value LTE, <<~EOQ
          Field is less than or identical to the given value input.
          Behavior depends on the underlying type of the field
          and may not work as expected for some fields.
        EOQ
        value MATCHES, "Field contains input as a subset"
        value NOT_MATCHES, "Field does not contain input as a subset"
        value NULL, "Field is null"
        value NOT_NULL, "Field is not null"
      end

      class FilterRule < ::GraphQL::Schema::InputObject
        description "A rule for filtering the results of a search"

        # TODO: fields should be restricted to what is exposed in the original type
        argument :field, String, required: true, description: "Field on which to filter. Should be an attribute on the type being queried"
        argument :filter_type, FilterType, required: true, description: "Type of filter to apply"
        argument :filter_values, [String], required: false, description: <<~EOD
          Which values to use as an input to the filter.
          Not required or used when the filter type is NULL or NOT_NULL.
          When multiple values are provided, results remain in the result set if they
          match any of the input values.
        EOD
      end

      # GraphQL field extension class which adds dynamic filtering to any
      # field which is backed by an ActiveRecord::Relation.
      #
      # Adds a `filter_rules` argument to the type in the schema,
      # and automatically builds SQL filter clauses based on the input
      # provided in the GraphQL document.
      # Composes nicely with Relay connections with no additional configuration.
      class FilterableExtension < ::GraphQL::Schema::FieldExtension
        ARGUMENT_FIELD = :filter_rules
        include ActiveRecordWrapperFieldExtension

        def apply
          field.argument argument_field, [FilterRule], required: false, description: <<~EOD
            Rules for filtering the results of a search.
            When multiple filter rules are provided, results are included only if they match
            all provided filter rules.
          EOD
        end

        private

        def with_ar_condition(relation, field:, filter_type: nil, filter_values: nil)
          relation_field = relation_field_from_input(relation: relation, field: field)

          case filter_type
          when FilterType::EQ
            relation.where(relation_field => filter_values)
          when FilterType::NE
            relation.where(build_generic_or(relation_field, "%s != ?", filter_values))
          when FilterType::GT
            relation.where(build_generic_or(relation_field, "%s > ?", filter_values))
          when FilterType::LT
            relation.where(build_generic_or(relation_field, "%s < ?", filter_values))
          when FilterType::GTE
            relation.where(build_generic_or(relation_field, "%s >= ?", filter_values))
          when FilterType::LTE
            relation.where(build_generic_or(relation_field, "%s <= ?", filter_values))
          when FilterType::MATCHES
            relation.where(build_generic_or(relation_field, "%s like ?", filter_values.map { |v| "%#{v}%" }))
          when FilterType::NOT_MATCHES
            relation.where(build_generic_or(relation_field, "%s not like ?", filter_values.map { |v| "%#{v}%" }))
          when FilterType::NULL
            relation.where(relation_field => nil)
          when FilterType::NOT_NULL
            relation.where.not(relation_field => nil)
          else
            raise NoMethodError, "No implementation for #{filter_type}"
          end
        end

        # Builds a disjunction from an arbitrary list of match values
        # and a predicate, expressed as a string template
        #
        # Build a list [predicate_string, bind0, bind1, ... bindN] by
        # looping through the values to match against and appending
        # more " OR (...)" queries to the predicate and plopping
        # the new value on the end.
        #
        # You'd think that ActiveRecord or arel would have a generic way of
        # constructing an `OR` query, but you'd be wrong.
        def build_generic_or(relation_field, template_string, filter_values)

          # Dear future people: the base case of an arbitrary disjunction
          # is "false"
          # (in other words, if there are no filter values, the resulting
          # predicate will match nothing.
          # One rule with no values IS NOT THE SAME as no rules)
          #
          # Don't try to change this or I will haunt you.
          or_clause_base_case = "(1 = 0)"

          filter_values.reduce([or_clause_base_case]) do |acc, value|
            predicate, *binds = acc
            [predicate + sprintf(" or (#{template_string})", relation_field), *binds, value]
          end
        end
      end
    end
  end
end
