# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/utils/account_utils'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'hitimes'
require 'user_management/utils/event_publish'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class AddUserGroup < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront', 'alert', 'user_management.kafka_message']
          include ::Platform::Helpers::Wavefront
          include UserManagement::Account::Utils
          include UserManagement::Utils::EventPublish

          argument :name, String, required: true
          argument :description, String, required: false
          argument :identity_attribute_rules, [UserGroupIdentityAttributeRuleInput], required: true

          field :user_group, UserGroup, null: false

          def resolve(name:, description: nil, identity_attribute_rules:)
            wf_time('user_management.add_user_group.resolve') do
              is_service_account = service_account?(context)
              logger.info("addUserGroup (is_service_account: #{is_service_account})")
              if context[:request].current_organization.nil? || context[:request].current_organization <= 0
                logger.error("addUserGroup: No organization provided")
                raise ::GraphQL::ExecutionError, "An organization is required"
              end

              user = nil
              unless is_service_account
                if context[:request].user_crn.nil?
                  logger.error("addUserGroup: No user ID provided")
                  raise ::GraphQL::ExecutionError, "An user ID is required to execute this request"
                end

                UserManagement::Authorizer.authorize!(
                  subject: "user_group",
                  actions: "create",
                  context: context)
                user = context[:request].user_crn.user
              end

              logger.info("creating group called: #{name}")
              new_group = ActiveRecord::Base.transaction do
                Stores::ArModels::UserGroup.create!(
                  customer_id: context[:request].current_customer,
                  organization_id: context[:request].current_organization,
                  created_by_id: user,
                  updated_by_id: user,
                  name: name,
                  description: description,
                  identity_attribute_rules: identity_attribute_rules.map do |rule_input|
                    Stores::ArModels::IdentityAttributeRule.new(
                      customer_id: context[:request].current_customer,
                      rule_key: rule_input[:key],
                      identity_attribute_rule_values: rule_input[:values].map do |rule_value_input|
                        Stores::ArModels::IdentityAttributeRuleValue.new(rule_value: rule_value_input,
                                                                         customer_id: context[:request].current_customer)
                      end
                    )
                  end
                )
              end

              publish(new_group, context, ::UserManagement::KafkaMessage::ADD)

              { user_group: new_group }
            end
          end
        end
      end
    end
  end
end
