# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'hitimes'
require 'user_management/utils/event_publish'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class AddUsersToUserGroup < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront','alert', 'user_management.kafka_message', 'clients.privilege_escalation_checker']
          include ::Platform::Helpers::Wavefront
          include UserManagement::Account::Utils
          include UserManagement::Utils::EventPublish

          argument :user_group_id, Types::CRN, required: true
          argument :user_ids, [Types::CRN], required: true

          field :user_group, UserGroup, null: false

          def resolve(user_group_id:, user_ids:)
            wf_time('user_management.add_users_to_user_group.resolve') do

              is_service_account = service_account?(context)
              logger.info("addUsersToGroup (is_service_account: #{is_service_account})")
              logger.info("addUsersToGroup user_group_id: #{user_group_id}, user_ids: #{user_ids}")
              if context[:request].current_organization.nil? || context[:request].current_organization <= 0
                logger.error "addUsersToGroup: No organization provided"
                raise ::GraphQL::ExecutionError, "An organization is required"
              end

              UserManagement::Authorizer.authorize!(subject: "user_group", actions: "update", context: context)

              existing_group = Stores::ArModels::UserGroup.where(
                customer_id: context[:request].current_customer,
                organization_id: context[:request].current_organization,
                id: user_group_id.details.to_i
              ).first
              if existing_group.nil?
                raise ::GraphQL::ExecutionError, "Record not found"
              end

              if is_service_account
                logger.info("Bypassing privilege escalation check for service account for addUsersToGroup user_group_id: #{user_group_id}")
              else
                logger.info("Checking privilege escalation for update action on user_group_id: #{user_group_id}")
                if privilege_escalation_checker.privilegeEscalated(subject: user_group_id, action: "update", context: context)
                  logger.error("Privilege escalation occurred for update action on user_group_id: #{user_group_id} for user: #{context[:request].current_user}")
                  raise ::GraphQL::ExecutionError, "Privilege escalation occurred for update action on user_group_id: #{user_group_id}"
                end
              end

              users = Stores::ArModels::User.where(
                customer_id: context[:request].current_customer,
                id: user_ids.map(&:details).map(&:to_i)
              )
              # Need to assign these individually due to =
              # operator overload in User.rb
              number_of_user_before_update = existing_group.users.size()
              ActiveRecord::Base.transaction do
                users.each do |user|
                  # Do not allow service accounts to be removed via API
                  raise ::GraphQL::ExecutionError, "Service Integration Users may not be directly added to user groups: #{user_group_id}" if user && user.service_account?
                  existing_group.users << user
                end
                number_of_user_after_update = existing_group.users.size()
                if(number_of_user_after_update != number_of_user_before_update)
                  existing_group.touch
                end
                existing_group.save!
                publish(existing_group, context)
              end

              { user_group: existing_group }
            end
          end
        end
      end
    end
  end
end
