# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'user_management/stores/shard_mixin'
require 'user_management/errors'
require 'hitimes'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class ApproveIntegrationUser < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront', 'alert', 'service_integration_accounts', 'clients.role_management', :audit]
          include ::Platform::Helpers::Wavefront
          include UserManagement::Account::Utils
          include ::UserManagement::Stores::ShardMixin

          argument :id, Types::CRN, required: true

          field :id, ID, null: false

          def resolve(id:)
            logger.info("ApproveIntegrationUser #{id} - #{service_integration_accounts} - #{context[:request].current_customer}")
            # Authorize
            UserManagement::Authorizer.authorize!(subject: "service_integration", actions: "approve", context: context)
            # Find the service integration by id
            service_integration = lookup_service_integration(id, service_integration_accounts)
            # Lookup customer on customer shard, it should be there if this is being called
            customer = Stores::ArModels::Customer.where(id: context[:request].current_customer).first
            # initial validate
            validate(customer, context[:request].current_organization, service_integration)
            # Validate not already approved
            email_address = generate_email_address(service_integration['name'], context[:request].current_customer)
            validate_not_approved(customer, email_address, service_integration['name'], service_integration['oauthAppId'])
            # Call actual user create (should throw error if customer not found)
            create_integration(service_integration['email'], service_integration['name'], customer, nil, nil, service_integration['oauthAppId'], service_integration['roleDocument'], context)
            logger.info("Found service integration #{service_integration}")
            # Audit the message
            audit.log("com.cloudhealthtech.audit.user-management.approve", context[:request], {:action => "approve", :id => id})
            { id: id.to_s }
          end

          # Create new user based on the info from the service integration.
          def create_integration(email, name, customer, provider = nil, uid = nil, oauth_app_id, roleDocument, context)
            logger.debug("Create Integration for #{email} - #{name} - #{customer} - #{provider} - #{uid}")
            # TODO: Validate all needed arguments are present
            user = create_user(email, name, customer, provider, uid, oauth_app_id)
            # Use user from context for 'created_by_id'
            created_by_user = context[:request].user_crn.user
            # Create Group
            user_group = create_user_group(name, user, created_by_user)
            # Create Role Binding
            create_role_binding(user, user_group, roleDocument, context)
          end

          private

          # Create new user based on the info from the IDP.
          # This user should be created in the same tenant as the current_customer,
          # not the users customer.
          def create_user(email, name, customer, provider, uid, oauth_app_id)
            # Create the customer on master shard first
            with_master do
              # TODO: This doesn't need to be an instance variable
              @user = Stores::ArModels::User.new.tap do |user|
                # Generate a unique email address
                user.email = generate_email_address(name, customer.id)
                user.name     = name
                user.customer = customer
                # Setting password to random hex. This technically cannot be decrypted, but password is never used for IDP users
                user.encrypted_password = SecureRandom.hex(10)
                user.provider = provider
                user.uid      = uid
                user.roles = [get_no_access_role]
                # Set API Key
                user.api_key = SecureRandom.uuid
                # Add SI settings
                user.service_account = true
                user.oauth_app_id = oauth_app_id
                # Add approval of TOS for CHAPI calls
                # I think the VMWare one is the only one actually set during the TOS acceptance flow
                # but both are checked at runtime to make sure one or the other was accepted.
                user.agree_to_license = true
                user.agree_to_license_date = Time.now
                user.agree_to_vmware_eula_date = Time.now
              end
              @user.save!
            end
            @user.replicate_to_customer_data_shard
            # Create default org assignment, this should already be in the scope of the customer shard
            # Make sure to explicitly run this on data shard even though entire mutation should already
            # be wrapped in the customer_shard contenxt
            customer.using_data_shard do
              # Classiic roles are not currently replicated in UMS so we need to lookup the user again
              # on the customer shard and assign it
              role = get_no_access_role
              user = Stores::ArModels::User.where(customer_id: customer.id, id: @user.id).first!
              user.roles = [role]
              user.organizations = [customer.default_organization]
              user.save!
            end
            @user          
          end

          # helper method so we can override this in unit tests
          def get_no_access_role
            Stores::ArModels::Role.where(customer_id: nil, name: 'zero-access').first!
          end

          # Create the user group, this group uses manual placement and has no identity attribute rules
          # and is created in the default organization / TLOU of the users customer
          def create_user_group(name, user, created_by_user)
            logger.info "Creating user group for #{name} user"
            description = "#{name} user group"
            new_group = ActiveRecord::Base.transaction do
                Stores::ArModels::UserGroup.create!(
                  customer_id: user.customer.id,
                  organization_id: user.customer.default_organization.id,
                  created_by_id: created_by_user,
                  updated_by_id: created_by_user,
                  name: name,
                  description: description,
                  identity_attribute_rules: []
                )
              end
            user.user_groups << new_group
            new_group
          end

          # Create the role binding, this should also fail in the role document 
          # doesn't exist
          def create_role_binding(user, user_group, role_document_id, context)
            logger.info "Creating role binding for #{user.id}"
            # Add cross tenant check if CRN of role document contains a tenant ID
            user_group_crn = "crn:#{user.customer.id}:user-group/#{user_group.id}"
            org_crn = "crn:#{user.customer.id}:organization/#{user.customer.default_organization.id}"
            role_binding = role_management.create_role_binding(user_group_crn, org_crn, role_document_id, context)
            logger.info "Role Binding created #{role_binding}"
            role_binding
          end
        end
       end
    end
  end
end
