# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'hitimes'
require 'user_management/utils/event_publish'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class DeleteUserGroup < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront', 'alert', 'user_management.kafka_message']
          include ::Platform::Helpers::Wavefront
          include UserManagement::Utils::EventPublish

          argument :id, Types::CRN, required: true

          field :id, ID, null: false

          def resolve(id:)
            wf_time('user_management.delete_user_group.resolve') do

              logger.info("deleteUserGroup id: #{id}")

              if context[:request].current_organization.nil? || context[:request].current_organization <= 0
                logger.error("deleteUserGroup: No organization provided")
                raise ::GraphQL::ExecutionError, "An organization is required"
              end

              UserManagement::Authorizer.authorize!(subject: "user_group", actions: "delete", context: context)

              existing_group = Stores::ArModels::UserGroup.where(
                customer_id: context[:request].current_customer,
                organization_id: context[:request].current_organization,
                id: id.details.to_i
              ).first
              if existing_group.nil?
                raise ::GraphQL::ExecutionError, "Record not found"
              end

              ActiveRecord::Base.transaction do
                existing_group.destroy
                wavefront.counter('user_group.deleted', 1, { customer_id: context[:request].current_customer.to_s } )
                publish(existing_group, context, ::UserManagement::KafkaMessage::DELETE)
              end

              { id: id.to_s }
            end
          end
        end
      end
    end
  end
end
