# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'hitimes'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class DeleteUserGroups < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront']
          include ::Platform::Helpers::Wavefront

          argument :ids, [Types::CRN], required: true

          field :ids, [ID], null: false

          def resolve(ids:)
            wf_time('user_management.delete_user_groups.resolve') do
              logger.info("deleteUserGroups ids: #{ids}")
              if context[:request].current_organization.nil? || context[:request].current_organization <= 0
                logger.error "deleteUserGroups: no organization provided"
                raise ::GraphQL::ExecutionError, "An organization is required"
              end

              UserManagement::Authorizer.authorize!(subject: "user_group", actions: "delete", context: context)

              logger.info("Selecting groups to delete")
              existing_groups = Stores::ArModels::UserGroup.where(
                customer_id: context[:request].current_customer,
                organization_id: context[:request].current_organization,
                id: ids.map(&:details).map(&:to_i)
              )
              if existing_groups.empty?
                raise ::GraphQL::ExecutionError, "Record not found"
              end

              logger.info "Destroying #{existing_groups.length} groups"
              ActiveRecord::Base.transaction do
                count = existing_groups.count
                existing_groups.destroy_all
                wavefront.counter('user_group.deleted', count, { customer_id: context[:request].current_customer.to_s } )
              end

              { ids: ids.map(&:to_s) }
            end
          end
        end
      end
    end
  end
end
