# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'hitimes'
require 'user_management/utils/event_publish'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class RemoveUsersFromUserGroup < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront', 'alert', 'user_management.kafka_message']
          include ::Platform::Helpers::Wavefront
          include UserManagement::Utils::EventPublish

          argument :user_group_id, Types::CRN, required: true
          argument :user_ids, [Types::CRN], required: true

          field :user_group, UserGroup, null: false

          def resolve(user_group_id:, user_ids:)
            wf_time('user_management.remove_users_from_user_group.resolve') do
              logger.info "removeUsersFromUserGroup user_group_id: #{user_group_id}, user_ids: #{user_ids}"
              if context[:request].current_organization.nil? || context[:request].current_organization <= 0
                logger.error "removeUsersFromUserGroup: no organization provided"
                raise ::GraphQL::ExecutionError, "An organization is required"
              end

              UserManagement::Authorizer.authorize!(subject: "user_group", actions: "update", context: context)

              logger.info "Fetching user group"
              existing_group = Stores::ArModels::UserGroup.where(
                customer_id: context[:request].current_customer,
                organization_id: context[:request].current_organization,
                id: user_group_id.details.to_i
              ).first
              if existing_group.nil?
                raise ::GraphQL::ExecutionError, "Record not found"
              end

              # TODO: privilege escalation mitigation
              ActiveRecord::Base.transaction do
                to_remove = user_ids.map(&:details).map(&:to_i).to_set
                logger.info "found #{to_remove.count} user to remove"
                number_of_user_before_update = existing_group.users.size()
                existing_group.users = existing_group.users.reject do |user|
                  # Do not allow service accounts to be removed via API
                  raise ::GraphQL::ExecutionError, "Service Integration Users may not be directly removed from user groups: #{user_group_id}" if user && user.service_account?
                  to_remove.include?(user.id)
                end
                number_of_user_after_update = existing_group.users.size()
                if(number_of_user_after_update != number_of_user_before_update)
                  existing_group.touch
                end
                existing_group.save!
                publish(existing_group, context)
              end

              { user_group: existing_group }
            end
          end
        end
      end
    end
  end
end
