# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'user_management/stores/shard_mixin'
require 'user_management/errors'
require 'hitimes'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class RevokeIntegrationUser < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront', 'alert', 'service_integration_accounts', :audit]
          include ::Platform::Helpers::Wavefront
          include UserManagement::Account::Utils
          include ::UserManagement::Stores::ShardMixin

          argument :id, Types::CRN, required: true

          field :id, ID, null: false

          def resolve(id:)
            logger.info("RevokeIntegrationUser #{id} - #{service_integration_accounts} - #{context[:request].current_customer}")
            # Authorize
            UserManagement::Authorizer.authorize!(subject: "service_integration", actions: "revoke", context: context)
            # Find the service integration by id 
            service_integration = lookup_service_integration(id, service_integration_accounts)
            # Lookup customer on customer shard, it should be there if this is being called
            customer = Stores::ArModels::Customer.where(id: context[:request].current_customer).first
            # initial validate
            validate(customer, context[:request].current_organization, service_integration)
            # Validate not already approved
            email_address = generate_email_address(service_integration['name'], context[:request].current_customer)
            validate_approved(customer, email_address, service_integration['name'], service_integration['oauthAppId'])
            # Call actual user delete (should throw error if customer not found)
            remove_integration(id, service_integration['oauthAppId'], customer)
            logger.info("Found service intregration #{service_integration}")
            # Audit the message
            audit.log("com.cloudhealthtech.audit.user-management.revoke", context[:request], {:action => "revoke", :id => id})
            { id: id.to_s }
          end

          # Remove the integration based on the info from the service integration.
          def remove_integration(id, oauth_app_id, customer)
            logger.debug("Remove Integration for #{id} - #{customer.to_json}")
            # we should be able to get away with just deleting the user and user group.
            # Kafka should take care of sending a delete to RMS, and RB stays in place regardless
            remove_user_group(id, oauth_app_id, customer)
            remove_user(id, oauth_app_id, customer)
          end

          private

          def remove_user_group(id, oauth_app_id, customer)
            # Lookup and destroy on customer shard
            user = find_user(id, oauth_app_id, customer)
            # Sanity check that we only find one
            raise ::GraphQL::ExecutionError, "Integration user belongs to more than one user group" unless user.user_groups.count == 1
            # TODO Add sanity check group contains only 1 user and that user has the same oauthAppId

            user.user_groups[0].destroy
          end

          def remove_user(id, oauth_app_id, customer)
            # Find the user for the service acccount
            user = find_user(id, oauth_app_id, customer)
            # TODO: This is wrapped in the customer shard already
            logger.debug("Removing user for service integration #{user.to_json}")
            # Destroy on customer shard
            user.destroy
            # Destroy on master shard
            with_master do
              u = find_user(id, oauth_app_id, customer)
              u.destroy
            end
          end

          def find_user(_id, oauth_app_id, customer)
            Stores::ArModels::User.where(customer_id: customer.id, service_account: true, oauth_app_id: oauth_app_id, is_deleted: false).first!
          end
        end
      end
    end
  end
end
