# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'hitimes'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class UpdateUserGroup < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront']
          include ::Platform::Helpers::Wavefront
          include RequestValidator

          argument :id, Types::CRN, required: true
          argument :name, String, required: true
          argument :description, String, required: false
          argument :identity_attribute_rules, [UserGroupIdentityAttributeRuleInput], required: true

          field :user_group, UserGroup, null: false

          def resolve(id:, name:, description: nil, identity_attribute_rules:)
            wf_time('user_management.update_user_group.resolve') do
              logger.info("updateUserGroup id: #{id}, name: #{name}, description: #{description}, attrs: #{identity_attribute_rules}")
              validate_request_has_organization!
              validate_request_has_user!

              UserManagement::Authorizer.authorize!(subject: "user_group", actions: "update", context: context)

              logger.info("fetch group")
              existing_group = Stores::ArModels::UserGroup.where(
                customer_id: context[:request].current_customer,
                organization_id: context[:request].current_organization,
                id: id.details.to_i
              ).first
              if existing_group.nil?
                raise ::GraphQL::ExecutionError, "Record not found"
              end

              current_user = context[:request].current_user
              existing_group_customer_id = existing_group.customer_id

              existing_group.update_all_attributes!(
                updated_by_user_id: current_user,
                name: name,
                description: description,
                identity_attribute_rules: identity_attribute_rules,
                customer_id: existing_group_customer_id
              )

              { user_group: existing_group }
            end
          end
        end
      end
    end
  end
end
