# frozen_string_literal: true

require 'graphql'
require 'user_management/stores/ar_models'
require 'user_management/utils/account_utils'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'user_management/stores/shard_mixin'
require 'user_management/errors'
require 'hitimes'

module UserManagement
  module GraphQL
    module Types
      module Mutations
        class UpsertFederatedUser < ::GraphQL::Schema::RelayClassicMutation
          include Import['logger', 'wavefront']
          include ::Platform::Helpers::Wavefront
          include UserManagement::Account::Utils
          include ::UserManagement::Stores::ShardMixin

          argument :customer_id, Integer, required: true
          argument :email, String, required: true
          argument :name, String, required: true
          argument :uid, String, required: false
          argument :provider, String, required: true
          argument :role, String, required: true
          argument :organization, String, required: false
          argument :idp_raw_data, String, required: true

          field :federated_user, FederatedUser, null: false

          def resolve(customer_id:, email:, name:, uid:, provider:, role:, organization:, idp_raw_data:)
            wf_time('user_management.upsert_federated_user.resolve') do
              hash_from_idp = JSON.parse(idp_raw_data)
              is_service_account = service_account?(context)
              logger.info("upsertFederatedUser (is_service_account: #{is_service_account})")
              unless is_service_account
                raise ::GraphQL::ExecutionError, "Only service accounts are authorized to perform this operation"
              end

              redacted_info = hash_from_idp.slice('roles', 'groups')
              hash_from_idp.keys.each do |key|
                redacted_info[key] = '**redacted**' unless redacted_info.key?(key)
              end

              logger.info("upsertFederatedUser: #{customer_id}, #{email}, #{name}, #{uid}, #{provider}, #{role}, #{organization}, #{redacted_info}")

              customer = Stores::ArModels::Customer.where(id: customer_id.to_s).first
              if customer.nil?
                raise ::GraphQL::ExecutionError, "Customer with id #{customer_id} not found"
              end

              # NOTE: This is opposite of CloudPercept which looks on master first
              @user = Stores::ArModels::User.where(email: email, customer_id: customer_id.to_s).first
              if @user.nil?
                logger.info("Cannot find user #{email}. Creating a new user for customer #{customer.id}")
                create_user(email, name, customer, provider, uid)
              else
                logger.info("User found #{email}. Updating an existing user for customer #{customer.id}")
                update_user(name, provider, uid)
              end

              # Check if Customer TLOU is classic
              default_org_classic = customer.using_data_shard do
                !customer.default_organization.flex_org?
              end

              # Currently we are setting this role to customer_admin_user in order to make sure Partner User are onboarded via CSP
              role = 'customer_admin_user' if default_org_classic && customer.is_msp? && role == 'ariacost:administrator'

              assign_sso_roles_and_membership(role, customer)

              ::UserManagement::Actions::UpdateIdentityAttributes.new.call(customer_id, @user.id, hash_from_idp)

              logger.info("Upserted user [#{@user.id}] was placed into groups by IDP rules [#{@user.user_groups.to_a}]")

              update_user_organization(customer, organization)

              role_id = @user.roles.first.id if @user.roles.present?
              org_id = @user.organizations.first.id if @user.organizations.present?

              {
                  federated_user: {
                      id: @user.id,
                      name: @user.name,
                      email: @user.email,
                      customer_id: customer.id,
                      uid: @user.uid,
                      provider: @user.provider,
                      role_id: role_id,
                      organization_id: org_id,
                      idp_raw_data: idp_raw_data
                  }
              }
            end
          end

          # Assign SSO Roles and FlexOrgs UserGroup Memberships
          def assign_sso_roles_and_membership(role, customer)
            with_master do
              @user.set_sso_role(role, customer)
              @user.save!
            end
            @user.replicate_to_customer_data_shard
          end

          # Create new user based on the info from the IDP.
          def create_user(email, name, customer, provider, uid)
            with_master do
              @user = Stores::ArModels::User.new.tap do |user|
                user.email    = email
                user.name     = name
                user.customer = customer
                # Setting password to random hex. This technically cannot be decrypted, but password is never used for IDP users
                user.encrypted_password = SecureRandom.hex(10)
                user.provider = provider
                user.uid      = uid
              end
              @user.save!
            end
            @user.replicate_to_customer_data_shard
          end

          # Update user based on info from the IDP
          def update_user(name, provider, uid)
            with_master do
              @user.name       = name
              @user.provider   = provider
              @user.uid        = uid
              @user.is_deleted = false
              # User may have changed roles. Clear out previous and set current.
              @user.roles = []
              @user.save!
            end
            @user.replicate_to_customer_data_shard
          end

          # Update users organization
          def update_user_organization(customer, organization)
            if organization
              preferred_org = customer.using_data_shard do
                Stores::ArModels::Organization.where(customer_id: customer.id, sso_uid: organization).first
              end
              if preferred_org.nil?
                raise ::GraphQL::ExecutionError, "Preferred Organization #{organization} not found"
              end
            end
            org = @user.default_organization(context, preferred_org)

            logger.info("default_organization : #{org}")
            
            if org.nil? && customer.is_msp? && (customer.sso_provider == 'csp' || customer.sso_provider == 'csp_pkce')
              # we need this explicit org for msp customer onboarding via csp path
              org =  UserManagement::Stores::ArModels::Organization.where(customer_id: customer.id, default_organization: true).first
              logger.info "Default org for msp customer: #{org}"
            end
            
            return unless org

            logger.info("Found default organization #{org.id} for user #{@user.email}")
            customer.using_data_shard do
              @user.organizations = [org]
              @user.save!
            end
          end

        end
      end
    end
  end
end
