# frozen_string_literal: true

require 'graphql'
require 'user_management/utils/account_utils'
require 'user_management/stores/ar_models'
require 'user_management/graph_ql/types/user_idp'
require 'user_management/graph_ql/types/user_exists_payload'
require 'user_management/graph_ql/types/user_default_organization'
require 'user_management/graph_ql/types/user_default_organization_id_input'
require 'user_management/graph_ql/types/resolvers/idp_attribute_resolver'
require 'user_management/graph_ql/types/resolvers/default_organization_resolver'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/wavefront'
require 'hitimes'
require 'user_management/models/service_integration_user'

module UserManagement
  module GraphQL
    module Types
      class Query < ::Platform::GraphQL::Types::BaseObject
        include Import['logger', 'wavefront', 'service_integration_accounts']
        include UserManagement::Account::Utils
        include ::Platform::Helpers::Wavefront

        description "Query root"

        add_field(::GraphQL::Types::Relay::NodeField)
        add_field(::GraphQL::Types::Relay::NodesField)

        field :user_by_id, User, null: true do
          argument :id, CRN, required: true
        end

        class UserEdgeType < ::GraphQL::Types::Relay::BaseEdge
          node_type(User)
        end

        class UserConnectionType < ::GraphQL::Types::Relay::BaseConnection
          edge_type(UserEdgeType)
          field :total_count, Integer, null: false

          def total_count
            # object.nodes contains the list of users
            object.nodes&.count || 0
          end
        end

        field :users, UserConnectionType, extensions: [SortableExtension, FilterableExtension],connection: true, null: false do
          description "Get all users available within the current authorization context"
        end

        def user_by_id(id:)
          wf_time('user_management.query.user_by_id') do
            logger.info("userbyId #{id}")
            UserManagement::Authorizer.authorize!(
              subject: "user",
              actions: "read", context: context)

            logger.info("IsAuthorized for userById #{id} : true")
            logger.info("Fetching user from database for #{id}")
            Stores::ArModels::User.where(
              customer_id: context[:request].current_customer,
              id: id.details.to_i
            ).first
          end
        end

        def users
          wf_time('user_management.query.users') do
            logger.info("users query")
            UserManagement::Authorizer.authorize!(
              subject: "user",
              actions: "read", context: context)

            logger.info("fetching users")
            Stores::ArModels::User.where(
              customer_id: context[:request].current_customer,
              is_deleted: false
            )
          end
        end

        field :user_groups, UserGroup.connection_type, extensions: [SortableExtension, FilterableExtension], null: false do
          description "Get all user groups available within the current authorization context"
        end

        def user_groups
          wf_time('user_management.query.user_groups') do
            logger.info("userGroups")
            is_service_account = service_account?(context)
            unless is_service_account
              UserManagement::Authorizer.authorize!(
                subject: "user_group",
                actions: "read", context: context)
            end

            logger.info("fetching user groups (is_service_account = #{is_service_account})")
            rel = Stores::ArModels::UserGroup.where(
              customer_id: context[:request].current_customer
            )
            rel = rel.where(organization_id: context[:request].current_organization) unless is_service_account
            rel
          end
        end

        field :user_idp, UserIdPPayload, null: false do
          description "Get the identity attributes for the user"
        end

        def user_idp
          wf_time('user_management.query.user_idp') do
            logger.info("userIdp")
            UserManagement::Authorizer.authorize!(
              subject: "user",
              actions: "read", context: context)

            @user_idp ||= Resolvers::IdpAttributeResolver.new
          end
        end

        field :user_default_organization, UserDefaultOrganizationPayload, "Get the users default organization", null: false do
          argument :input, UserDefaultOrganizationInput, "A user CRN", required: true
        end

        def user_default_organization(value)
          wf_time('user_management.query.user_default_organization') do
            logger.info("userDefaultOrganization")
            UserManagement::Authorizer.authorize!(
              subject: "user",
              actions: "read", context: context)

            @user_default_organization ||= Resolvers::DefaultOrganizationResolver.new(value[:input], context)
          end
        end

        field :is_user_in_platform, UserExistsPayload, null: false do
          argument :email, String, required: true
        end

        def is_user_in_platform(value)
          wf_time('user_management.query.user_default_organization') do
            logger.info("isUserInPlatform #{value[:email]}}")
            is_service_account = service_account?(context)
            logger.info("isUserInPlatform (is_service_account: #{is_service_account})")
            unless is_service_account
              raise ::GraphQL::ExecutionError, "Only service accounts are authorized to perform this query"
            end
            # Do lookup here.
            Resolvers::PlatformUserResolver.new(value[:email], context)
          end
        end

        field :service_integration_users, ServiceIntegrationUser.connection_type, null: false do
          description "Get all service integration users available within the current authorization context"
        end

        def service_integration_users
          wf_time('user_management.query.service_integrations') do
            # Authorize
            UserManagement::Authorizer.authorize!(subject: "service_integration", actions: "read", context: context)
            # Lookup service integrations here
            accounts = []            
            service_integrations = service_integration_accounts&.each do |service_integration|
              sia = ::UserManagement::Actions::FindIntegrationUser.new.call(service_integration, context[:request].current_customer)
              accounts << sia
            end
            logger.info("Returned integrations #{accounts}")
            accounts
          end
        end
      end
    end
  end
end
