# frozen_string_literal: true

require 'graphql'

module UserManagement
  module GraphQL
    module Types
      class SortDirection < ::GraphQL::Schema::Enum
        value "ASC", "Ascending sort order"
        value "DESC", "Descending sort order"
      end

      class SortRule < ::GraphQL::Schema::InputObject
        # TODO: fields should be restricted to what is exposed in the original type
        argument :field, String, required: true, description: "Field on which to sort. Should be an attribute on the type being queried"
        argument :direction, SortDirection, required: false, default_value: "ASC", description: "Sort direction, ascending by default"
      end

      # GraphQL field extension class which adds dynamic sorting to any
      # field which is backed by an ActiveRecord::Relation.
      #
      # Adds a `sort_rules` argument to the type in the schema,
      # and automatically builds SQL `order by` clauses based on the input
      # provided in the GraphQL document.
      # Composes nicely with Relay connections with no additional configuration.
      class SortableExtension < ::GraphQL::Schema::FieldExtension
        ARGUMENT_FIELD = :sort_rules
        include ActiveRecordWrapperFieldExtension

        def apply
          field.argument argument_field, [SortRule], required: false, description: <<~EOD
            Rules for sorting search results.
            When multiple sort rules are provided, the first rule takes precendence
            and subsequent rules are used when the first rule does not provide a strict ordering.
          EOD
        end

        private

        def with_ar_condition(relation, field:, direction:)
          relation_field = relation_field_from_input(relation: relation, field: field)
          relation.order(relation_field => direction)
        end
      end
    end
  end
end
