# frozen_string_literal: true

require 'cht-lib-fabric/graphql/federation/types'
require 'user_management/utils/account_utils'

module UserManagement
  module GraphQL
    module Types
      class UserGroupEdgeWithMembershipSourceType < ::GraphQL::Types::Relay::BaseEdge
        node_type(UserGroup)

        field :membership_source, MembershipSource, null: false do
          description MEMBERSHIP_SOURCE_DESCRIPTION
        end

        def membership_source
          # While memory-inefficient, `.find()` is preferred over
          # `where().first` here as the latter would cause an N+1 query issue.
          # All user_groups_users are eager-loaded, we can avoid further queries.
          object.node.user_groups_users
            .find { |x| x.user_id == object.parent.id }
            .membership_source
        end
      end

      class UserGroupConnectionWithMembershipSourceType < ::GraphQL::Types::Relay::BaseConnection
        edge_type(UserGroupEdgeWithMembershipSourceType)
      end

      class User < ::GraphQL::Schema::Object
        implements ::GraphQL::Relay::Node.interface
        extend ::UserManagement::Account::Utils
        global_id_field :id

        field :id, ID, null: false
        field :email, String, null: false
        field :name, String, null: true
        field :last_activity, ::GraphQL::Types::ISO8601DateTime, null: true
        field :last_sign_in, ::GraphQL::Types::ISO8601DateTime,  null: true
        field :service_account, ::GraphQL::Types::Boolean, null: false
        field :oauth_app_id, ::GraphQL::Types::String, null: true

        def last_activity
          object.last_request_at
        end

        def last_sign_in
          object.current_sign_in_at
        end

        field :user_groups, UserGroupConnectionWithMembershipSourceType, null: false,
          extensions: [SortableExtension, FilterableExtension],
          # This is required when a connection class name doesn't end with Connection
          connection: true,
          resolve: -> (obj, _args, ctx) {
            # We eager-load all `user_groups_users` if necessary (i.e. if
            # looking up fields from the association) to avoid N+1 queries
            edges = ctx.irep_node
              .ast_nodes.find { |child| child.name == 'userGroups' }
              .children.find(-> { false }) { |child| child.name == 'edges' }
            requires_eager_loading = edges &&  edges.children.map(&:name).include?('membershipSource')

            is_service_account = service_account?(ctx)

            if requires_eager_loading
              rel = obj.user_groups.includes(:user_groups_users)
              rel = rel.where(organization_id: ctx[:request].current_organization) unless is_service_account
              rel
            else
              rel = obj.user_groups
              rel = rel.where(organization_id: ctx[:request].current_organization) unless is_service_account
              rel
            end
          } do
            description "Set of user groups that this user belongs to."
          end
      end
    end
  end
end
