# frozen_string_literal: true

require 'graphql'
require 'cht-lib-fabric/crns'

module UserManagement
  module GraphQL
    module Types
      # `UserEdgeWithMembershipSourceType` below refers to `User` which has not
      # been auto-loaded yet. Here we pre-define a User class that the edge
      # class can use, and we will fill in the body declaration in `user.rb`
      # when it gets loaded.
      # See https://stackoverflow.com/a/37499523/1935861
      class User < ::GraphQL::Schema::Object
      end

      class UserEdgeWithMembershipSourceType < ::GraphQL::Types::Relay::BaseEdge
        node_type(User)

        field :membership_source, MembershipSource, null: false do
          description MEMBERSHIP_SOURCE_DESCRIPTION
        end

        def membership_source
          # While memory-inefficient, `.find()` is preferred over
          # `where().first` here as the latter would cause an N+1 query issue.
          # All user_groups_users are eager-loaded, we can avoid further queries.
          object.node.user_groups_users
            .find { |x| x.user_group_id == object.parent.id }
            .membership_source
        end
      end

      class UserConnectionWithMembershipSourceType < ::GraphQL::Types::Relay::BaseConnection
        edge_type(UserEdgeWithMembershipSourceType)
        field :count, Integer, null: false

        def count
          # `object.nodes` is the collection of users
          object.nodes&.count
        end
      end

      class UserGroup < ::GraphQL::Schema::Object
        implements ::GraphQL::Relay::Node.interface
        global_id_field :id

        field :id, ID, null: false
        field :name, String, null: false
        field :organization_id, ID, null: false
        field :description, String, null: true
        field :created_at, ::GraphQL::Types::ISO8601DateTime, null: false
        field :updated_at, ::GraphQL::Types::ISO8601DateTime, null: false
        field :created_by_user_id, String, null: false
        field :updated_by_user_id, String, null: false
        field :users, UserConnectionWithMembershipSourceType, null: false,
          extensions: [SortableExtension, FilterableExtension],
          # This is required when a connection class name doesn't end with Connection
          connection: true,
          resolve: -> (obj, _args, ctx) {
            # We eager-load all `user_groups_users` if necessary (i.e. if
            # looking up fields from the association) to avoid N+1 queries
            edges = ctx.irep_node
              .ast_nodes.find { |child| child.name == 'users' }
              .children.find(-> { false }) { |child| child.name == 'edges' }
            requires_eager_loading = edges &&
              edges.children.map(&:name).include?('membershipSource')

            if requires_eager_loading
              obj.users.includes(:user_groups_users)
            else
              obj.users
            end
          } do
            description "Set of users belonging to this user group."
          end
        field :identity_attribute_rules, [UserGroupIdentityAttributeRule], null: false

        def organization_id
          ::Platform::CRNs::Organization.new(
            organization: object.organization_id,
            customer: object.customer_id
          )
        end

        def created_by_user_id
          ::Platform::CRNs::User.new(
            user: object.created_by ? object.created_by.id : 0,
            customer: object.created_by ? object.created_by.customer_id : 0
          )
        end

        def updated_by_user_id
          ::Platform::CRNs::User.new(
            user: object.updated_by ? object.updated_by.id : 0,
            customer: object.updated_by ? object.updated_by.customer_id : 0
          )
        end

        def identity_attribute_rules
          # TODO: fix n+1 query and add with_customer_shard as middleware
          object.identity_attribute_rules.map do |rule|
            {
              key: rule.rule_key,
              values: rule.identity_attribute_rule_values.map(&:rule_value)
            }
          end
        end
      end
    end
  end
end
