# frozen_string_literal: true

require 'active_record'
require 'graphql'
require 'user_management/graph_ql/types'
require 'cht-lib-fabric/crn'
require 'cht-lib-fabric/helpers/graphql'
require 'container'
require 'import'
require 'user_management/authorizer'
require 'cht-lib-fabric/helpers/graphql/metric_tracing'
require 'cht-lib-fabric/graphql/federation/types'
require 'user_management/actions/find_integration_user'
require 'user_management/utils/account_utils'

module UserManagement
  module GraphQL
    class Schema < ::GraphQL::Schema
      use ::Platform::Helpers::GraphQL::MetricTracing,
          client: Application[:wavefront]
      extend ::Platform::Helpers::GraphQL::RaiseError
      extend ActiveSupport::Inflector
      extend UserManagement::Account::Utils
      include ApolloFederation::Schema
      include ::Import[
        "user_management.operations.with_customer_shard",
        "service_integration_accounts"
      ]

      max_depth 13

      query Types::Query
      mutation Types::Mutation

      orphan_types [
        Types::User,
        Types::UserGroup
      ]

      # https://github.com/rmosolgo/graphql-ruby/issues/1109
      # We won't set a max page size by default since it makes pageInfo behave in an unexpected way
      # (even though it's evidently to spec: https://github.com/rmosolgo/graphql-ruby/pull/1623#issuecomment-402706803)
      #
      # This is a big pain and we should probably wrap or overwrite the BaseConnection class to make this better.
      # default_max_page_size 100

      def self.execute(query, args)
        with_customer_shard.call(context: args[:context]) do
          super(query, args)
        end
      end

      def self.with_customer_shard
        Application["user_management.operations.with_customer_shard"]
      end

      def self.logger
        Application["logger"]
      end

      def self.resolve_type(_type, obj, _ctx)
        # TODO: this should maybe defer to a method on the ArModel
        class_basename = demodulize(obj.class.name)
        begin
          Types.const_get(class_basename)
        rescue NameError => ex
          logger.error(ex)
          # UserManagement::GraphQL::Types::Foo is not a real symbol
          # The solution is to implement that Types class
          raise_error(
            ::Platform::ErrorCodes::INTERNAL_ERROR,
            "Type #{class_basename} does not have a corresponding type class"
          )
        end
      end

      # TODO: this should probably defer to a class method on an ArModel
      # that gets registered automatically on service startup
      def self.object_from_id(unique_id, context)
        crn = ::Platform::CRN.from_s(unique_id)
        service = crn.service
        validate_id_details(crn)

        if crn.customer > 0 && context[:request].current_customer != crn.customer
          raise_error(
            ::Platform::ErrorCodes::NOT_FOUND,
            "Tenant in CRN NOT owned by requested user"
          )
        end

        # Use action to lookup service integration accounts
        # Ideally SIA would be wrapped into an AR object at some point
        if service.eql?("service-integration")
          service_integration = lookup_service_integration(crn, Application["service_integration_accounts"])
          ::UserManagement::Actions::FindIntegrationUser.new.call(service_integration, context[:request].current_customer)
        else 
          class_basename = camelize(underscore(service))
          klass = begin
                    Stores::ArModels.const_get(class_basename)
                  rescue NameError => ex
                    logger.error(ex)
                    # UserManagement::Stores::ArModels::Foo is not a real symbol
                    # This is probably from a user requesting an object not in this service
                    raise_error(
                      ::Platform::ErrorCodes::INVALID_CRN_SERVICE,
                      "Type #{class_basename} does not have a corresponding model class"
                    )
                  end
          begin
            UserManagement::Authorizer.authorize!(
              subject: underscore(service),
              actions: "read",
              context: context)

            rel = klass
              .where(customer_id: context[:request].current_customer)
            # Only known support for organization_id is on UserGroup
            rel = rel.where(organization_id: context[:request].current_organization) if service.eql?('user-group')
            rel.find(crn.details.to_i)
          rescue ::ActiveRecord::RecordNotFound => ex
            raise_error(::Platform::ErrorCodes::NOT_FOUND, "#{ex} - #{crn}")
          end
        end
      end

      # This is NOT a complete implementation!
      # Just handles the simple cases where a model class maps 1-1 to a service name,
      # and there is always a customer in the CRN,
      # and the details section is just the ID
      # TODO: accept an override to this on the ArModel class
      def self.id_from_object(object, _type, context)
        ::Platform::CRN.new(
          service: dasherize(underscore(demodulize(object.class.name))),
          customer: context[:request].current_customer,
          details: object.id.to_s
        ).to_s
      end

      private

      # Check generically here that the details
      # is a valid integer, but the time this is called
      # the non-details of the CRN should have already
      # been validated
      def self.validate_id_details(crn)
        begin
          if crn.service.eql?("service-integration")
            # For Service Integration CRN's just make sure it has a value
            raise ArgumentError if crn.details.nil?
          else
            Integer(crn.details)
          end
        rescue ArgumentError => ex
          logger.error "Unable to parse CRN details for #{crn}"
          raise_error(::Platform::ErrorCodes::INVALID_CRN_FORMAT, crn)
        end
      end
    end
  end
end
