# frozen_string_literal: true

require 'dry/monads/result'
require 'cht-lib-fabric/error_codes'
require 'user_management/stores/shard_mixin'
require 'import'

module UserManagement
  module Operations
    class WithCustomerShard
      include ::Dry::Monads::Result::Mixin
      include Stores::ShardMixin
      include Import['logger', 'clients.shard_lookup']

      ASSET_DATA_SHARD_ENUM = "MAIN"

      def initialize(**kwargs)
        @synchronized_shards = {}
        super(**kwargs)
      end

      # An around step {https://dry-rb.org/gems/dry-transaction/around-steps/}
      # which wraps subsequent actions such that they use the customer shard
      def call(input, &_block)
        request = input[:tenant_scoped_request] || input[:context][:request]
        sa_lookup_context = { request: request }

        shard_id =
          shard_lookup
          .shard_assignment_by_data_store(
            ASSET_DATA_SHARD_ENUM,
            context: sa_lookup_context
          )
        db_shard = data_shard_alias(shard_id)

        with_data_shard(db_shard) do
          yield(Success(input))
        end
      end

      private

      # Do not allow two threads to get the same shard at the same time
      # in the same process
      # See https://github.com/jruby/activerecord-jdbc-adapter/pull/954
      #
      # This is a bit of a hack: it doesn't actually guarantee that
      # we don't concurrently evaluate the downstream code which is
      # not thread-safe, but it introduces enough of a jitter that
      # the downstream error occurs much, much less frequently, and
      # the jitter is very conservatively applied (first time a
      # particular shard is requested per process)
      # https://rollbar.com/cloudhealthtech/cht-svc-org-management/items/204/
      def data_shard_alias(shard_id)
        @synchronized_shards[shard_id] ||= Mutex.new.synchronize { "db#{shard_id}" }
      end
    end
  end
end
