# frozen_string_literal: true
require 'active_record'
require 'user_management/utils/model_replication'
require 'user_management/errors'

module UserManagement
  module Stores
    module ArModels
      class User < ::ActiveRecord::Base
        include ::UserManagement::Utils::ModelReplication
        include ::UserManagement::Stores::ShardMixin

        belongs_to :customer

        has_and_belongs_to_many :organizations, join_table: :organization_users

        has_many :user_groups_users
        has_and_belongs_to_many :roles, join_table: :users_roles

        scope :with_email_address, lambda { |email| joins(:customer).where(email: email).where('customers.is_deleted = 0') }

        # [ENG-47926] - Use this method to fetch the user record instead of 'with_email_address'
        # This is to scope the user to the tenant based on the customer id that was provided.
        scope :with_email_address_and_customer_id, lambda { |email, customerId| joins(:customer).where(email: email).where(customer_id: customerId).where('customers.is_deleted = 0') }

        validate :validate_user

        has_many :user_groups, through: :user_groups_users do
          def <<(group)
            proxy_association.owner.save! unless proxy_association.owner.persisted?
            proxy_association.owner.find_or_create_group(group).save!
            proxy_association.owner.reload
          end
        end

        def validate_user

          logger.info "Validating user #{id} with email #{email} for customer #{customer_id}"

          # Need to look on master shard since users are not replicated
          # to every secondary shard
          # existing = Multidb.use(:master) do
          existing = with_master do
            logger.info "Fetching User record with email and customer id"
            ar = User.with_email_address_and_customer_id(email, customer_id.to_s)
            # NOTE: don't use "persisted?" since it breaks replication to other shards
            logger.debug "User with email and customer found #{ar.to_a}"
            if id.to_i > 0
              ar.where('users.id != ?', id).count
            else
              # FYI... "select 42 != NULL;" resolves to "NULL" in mysql
              # So for new users (like during signup, when id is nil) we skip checking id
              ar.count
            end
          end

          logger.error "User with #{email} already exists for customer #{customer_id}" if existing > 0

          raise ::UserManagement::Exceptions::UserWithEmailAlreadyExists, "User with email #{email} already exists for customer #{customer_id}." if existing > 0

        end

        def user_groups=(groups_to_assign)
          new_groups = groups_to_assign - user_groups
          old_groups = user_groups - groups_to_assign
          new_groups.each do |user_group|
            find_or_create_group(user_group).save!
          end
          old_groups.each do |old_group|
            find_or_create_group(old_group).delete
          end
          reload
          groups_to_assign
        end

        def can_use_classic_org?(org)
          role = roles.first

          if (!role.nil? && !org.nil?) then
            logger.info("Checking whether user [#{id}] can use classic org [#{org.name}] in role [#{role.name}]")
          else
            logger.error("Unable to check classic user for [#{id}]")
          end

          # true if:
          # - a non nil org was given
          # - and the org is classic and the user has access to classic orgs
          res = org.present? &&
            (!org.flex_org? && role && role.name != 'zero-access')
          res
        end

        def find_or_create_group(group)
          query = { customer_id: customer_id,
            user_id: id,
            user_group_id: group.id,
            membership_source: :sso
          }
          Stores::ArModels::UserGroupsUser.where(query).where("? <> ?", :membership_source, :manual).first_or_initialize
        end

        #
        # Returns a boolean indicating if a flex org is
        # is in the list of supplied flex orgs. This is
        # slightly different in that it only considers
        # orgs with direct role bindings
        #
        def can_use_preferred_flex_org?(preferred_org, flex_orgs)
          return false if flex_orgs.nil?
          return false if flex_orgs.empty?
          !flex_orgs.detect { |florg| florg.id.to_i == preferred_org.id.to_i }.nil?
        end

        #
        # Returns a default / current organization for the user. 
        # @param - A GraphQL query context
        # @param - An optional preferred organization
        def default_organization(query_ctx = nil, preferred_organization = nil)
          return @default_organization if @default_organization

          # See if there is a preferred organization via IDP
          # This may be a Classic or Flex Org

          if preferred_organization
            logger.info "Preferred Organization Specified? [#{preferred_organization.id}] - [#{preferred_organization.name}]"
          else
            logger.info "No preferred organization specfied"
          end

          # get the classic org linkage for this user.
          # If the use expressed a preferred org, use that instead of the first org in the list
          # it should already have been validated that it exist by the time we get here.
          #
          # otherwise we'll just use the original route of selecting the first org in the list
          @default_organization = preferred_organization ? preferred_organization  : organizations.first

          unless @default_organization.nil?
            logger.info("Checking [#{@default_organization.name}] - [#{@default_organization.id}] for user [#{id}] classic usage")
          else
            logger.error("No initial default organization for checking classic usage for user [#{id}]")
          end

          if can_use_classic_org?(@default_organization)
            logger.debug("Using a classic organization: [#{@default_organization.id}]")
          else
            @context = query_ctx
            @default_organization = default_ee_organization(preferred_organization)
          end

          unless @default_organization.nil?
            logger.info("User [#{id}] was placed into default organization [#{default_organization.name}] - [#{@default_organization.id}]")
          else
            # If we still dont have a default organization and the user specified a preference
            # we'll default back the default org. This is to prevent a 'no default organization' when
            # checking a preferred organization agains a flex org w/no role bindings. This will also
            # help preserve the current behavior when not using a preferred organization
            if preferred_organization
              logger.error "User [#{id}] specified a preferred flex org but it had no direct role bindings, defaulting back to default organization"
              @default_organization = UserManagement::Stores::ArModels::Organization.where(customer_id: customer_id, default_organization: true).first
              logger.info("User [#{id}] was placed into default organization [#{default_organization.name}] - [#{@default_organization.id}]")
            else
              # else default back to the current behavior
              logger.error("User [#{id}] was not placed into a default organization")
            end
          end

          @default_organization
        rescue => ex
          logger.error("An error Occurred: #{ex.message}")
          nil
        end

        def ee_organizations
          return @ee_organizations if @ee_organizations.present?
          org_ids = get_org_ids(role_management_graphql)
          query = UserManagement::Stores::ArModels::Organization.where(customer_id: customer_id, id: org_ids)
          logger.debug("Building Organization Cache QUERY: #{query.to_sql}")
          @ee_organizations ||= query.all
        end

        #
        # Returns an EE/Flex Org that might be a candidate for the callers default organization.
        #
        # @param - An optional preferrred organization
        def default_ee_organization(preferred_organization = nil)

          # Assign once to avoid making multiple calls when no role bindings are present
          ee_orgs_to_consider = ee_organizations.to_a

          # Check if the user supplied a preferred organization and can access
          # if first. If they did and they can't access if then return nil
          if preferred_organization 
            return preferred_organization if can_use_preferred_flex_org?(preferred_organization, ee_orgs_to_consider)
          end

          logger.info "Considering default EE organizations for user [#{id}] - [#{ee_orgs_to_consider}]"

          # Only want to look at the orgs i'm in according to the RoleManagement Service
          org_ids = ee_orgs_to_consider.select{|o| o.flex_org?}.map(&:id)
          
          # We want to get the Highest Org in the Hierarchy
          high_ish_organizations = ee_orgs_to_consider.select do |org|
            # TODO: Decide when a user has chose an org as default

            # it is not a flexorg! This can happen when the root org is included but
            # we handle that elsewhere in this code: `can_use_classic_org?`
            next(false) unless org.flex_org?
            
            # It's the root node if the parent org is nil
            next(true) if org.parent_organization_id.nil? && org.flex_org?
            
            # It's not the highest if it's parent is in the list
            next(false) if org.parent_organization_id.in?(org_ids)
            true
          end
          logger.debug("Have #{high_ish_organizations.size} organizations")

          
          if high_ish_organizations.empty?
            logger.warn("No higher organizations")
          end
          
          highest_org = high_ish_organizations.min do |org1, org2|
            # Min By number of parents then name
            2 * (org1.parent_organizations.size <=> org2.parent_organizations.size) + (org1.name <=> org2.name)
          end

          if highest_org
            logger.info("Chose [#{highest_org.id}] for default ee organization for user [#{id}]")
          else
            logger.warn("No default ee organization chosen for user [#{id}]")
          end

          highest_org
        end

        def get_org_ids(user_groups)
          user_groups.map do |ug|
            ug['organizationIds'].map do |org_id|
              org_id.split('/').last
            end
          end.flatten.uniq
        end

        def role_management_graphql
          role_management = Application['clients.role_management']
          user_group_crns = user_groups.map(&:crnify)
          result = role_management.get_role_documents(user_group_crns, @context)
          logger.debug("Role Management Response: #{result.to_h}")
          response = result.to_h.with_indifferent_access
          # The service does not support filtering yet so do it here
          return [] if response['userGroupRoleBindings'].nil? || response['userGroupRoleBindings']['edges'].nil?
          response['userGroupRoleBindings']['edges'].map{|o| o['node']}.select{|n| n['userGroupId'].in?(user_group_crns)}
        end

        #
        # Set the SSO role of the user. This code is only called
        # through the upsertPath atm. This path will handle updating
        # the classic role to no-access for the upsert path for all users
        # Admin, managed or otherwise.
        #
        # But when we have hybrid tenants migrated to CSP in the future this
        # code may handle users that still have classic roles
        # in the TLOU.  The check in classic is a bit more 
        # restrictive as well in that it also checks if the tenant
        # is FlexOrgs only, which is true by default for CSP based tenants.
        #
        def set_sso_role(role, cust)
          case role
          when 'power_user'
            role = 'power'
          when 'standard_user'
            role = 'standard'
          when 'admin_user'
            role = 'admin' if cust.id == 1
          when 'customer_admin_user'
            role = 'customer_admin'
          when 'ariacost:administrator'
            role = 'zero-access'
          when 'ariacost:managed'
            role = 'zero-access'
          end
          # builtin roles and internal role are not associated to any customer
          unless ['zero-access', 'customer_admin', 'admin', 'standard', 'power'].include?(role)
            customer_role = Role.where(customer_id: cust.id, name: role).first
          else
            customer_role = Role.where(customer_id: nil, name: role).first
          end
          # self is necessary in the line below!
          self.roles = [customer_role] if customer_role
        end
      end
    end
  end
end
