# frozen_string_literal: true

require 'cht-lib-fabric/crn'
require 'user_management/stores/ar_models'

module UserManagement
  module Account
    module Utils
      def service_account?(context)
        return false if context[:request].jwt_claims.nil?
        service_account = begin
             subject = context[:request].jwt_claims[:sub]
             ::Platform::CRNs::ServiceAccount.from_s(subject)
           rescue ::Platform::InvalidCRNError
             false
           end
        service_account
      end

      #
      # Returns a service integration entry from the file.
      # This entry contains all the information to create the
      # various artifacts from.
      #
      def lookup_service_integration(id, service_integrations)
        service_integrations&.each do |service_integration|
          val = app_key = service_integration.keys[0]
          app = service_integration[app_key]
          logger.debug "Comparing id #{id} to app id #{app['id']} - #{app['id'].to_s == id.to_s }"
          return app if app['id'].to_s == id.to_s
        end
        logger.error("Unknown service integration id provided #{id}")
        raise ::GraphQL::ExecutionError, "Unknown service integration id provided #{id}"
      end

      # 
      # Various validations performed for service integration accounts
      #
      def validate(customer, current_organization, service_integrations)
        # This should run on the customer shard
        # Check TLOU
        org = ::UserManagement::Stores::ArModels::Organization.where(customer_id: customer.id, id: current_organization).first!
        raise ::GraphQL::ExecutionError, "Service Integration accounts may only be managed in the default organization" unless org.default_organization?
        # Make sure we're only in flex orgs
        raise ::GraphQL::ExecutionError, "Service Integration accounts may only be managed in flex organizations" unless org.flex_org
        # Check cross tenant for role documents
        role_document_tenant_id = ::Platform::CRN.from_s(service_integrations["roleDocument"]).customer
        unless role_document_tenant_id.nil? || role_document_tenant_id == 0
          raise ::GraphQL::ExecutionError, "Service Integration accounts may only use role documents from the same tenant" unless role_document_tenant_id == customer.id
        end
      end

      #
      # Validates that the account is not already approved by the
      # existence of a user for that account
      #
      def validate_not_approved(customer, email, name, oauth_app_id)
        users = Stores::ArModels::User.where(customer_id: customer.id, email: email, name: name, oauth_app_id: oauth_app_id, service_account: true, is_deleted: false).count
        raise ::GraphQL::ExecutionError, "Service Integration account has already been approved" unless users == 0 
      end

      #
      # Validates that the account is already approved by the
      # existence of a user for that account
      #
      def validate_approved(customer, email, name, oauth_app_id)
        users = Stores::ArModels::User.where(customer_id: customer.id, email: email, name: name, oauth_app_id: oauth_app_id, service_account: true, is_deleted: false).count
        raise ::GraphQL::ExecutionError, "Service Integration account has not been approved" unless users == 1 
      end

      #
      # Generate a unique email address based on the tenant ID and OAuthAppName
      #
      def generate_email_address(app_name, customer_id)
        munged_app_name = app_name.downcase.gsub(/\s+/, '') unless app_name.nil?
        "#{munged_app_name}-service-integration-user-#{customer_id}@vmware.com"
      end
    end
  end
end
